﻿Option Explicit On
Option Strict On

Imports System.Data
Imports System.Data.Sql
Imports System.Data.SqlClient
Imports Microsoft.VisualBasic

#Region " --- Employees (Employees) --- "

''' <summary>
''' Ermöglicht Datenzugriff auf die Tabelle Employees
''' </summary>
Public Class cSQLS_Employees
   Inherits cSQLS_Base

   ''' <summary>
   ''' Felder in der Tabelle (nur ein paar werden hier 
   ''' aufgeführt - für das Beispiel/der Einfachheit halber 
   ''' </summary>
   ''' <remarks></remarks>
   Public Enum eFields
      EmployeeID
      LastName
      FirstName
      Title
   End Enum

   ''' <summary>
   ''' Name der Tabelle
   ''' </summary>
   ''' <value></value>
   ''' <returns></returns>
   ''' <remarks></remarks>
   Friend ReadOnly Property TableName() As String
      Get
         Return "Employees"
      End Get
   End Property

   ''' <summary>
   ''' (Feld-) Name des PKs
   ''' </summary>
   ''' <value></value>
   ''' <returns></returns>
   ''' <remarks></remarks>
   Friend ReadOnly Property FieldName_RecordIdentifier() As String
      Get
         Return FieldName(eFields.EmployeeID)
      End Get
   End Property

   ''' <summary>
   ''' Gibt den Namen des angeforderten Feldes zurück.
   ''' </summary>
   ''' <param name="Field"></param>
   Public Function FieldName(ByVal Field As eFields) As String
      Return [Enum].GetName(GetType(eFields), Field)
   End Function

   ''' <summary>
   ''' Gibt eine Liste aller Datensätze zurück, z.B. um ein Grid oder eine Combobox zu befüllen.
   ''' </summary>
   Public Function Records_DataTable() As DataTable
      Dim fConnectionOpenedHere_CloseWhenExiting As Boolean = (DB_Connection.State <> ConnectionState.Open)
      Try
         Dim dt As New DataTable(Me.ToString)
         Dim cmd As New SqlCommand("p_Employees_RecordList", DB_Connection)
         cmd.CommandType = CommandType.StoredProcedure

         If fConnectionOpenedHere_CloseWhenExiting AndAlso DB_Connection.State <> ConnectionState.Open Then DB_Connection.Open()
         Dim da As New SqlDataAdapter(cmd)
         da.Fill(dt)

         'Den PK innerhalb der DataTable definieren (z.B. für Suchaktionen des Aufrufers)
         dt.PrimaryKey = New DataColumn() {dt.Columns(Me.FieldName(eFields.EmployeeID))}

         Return dt

      Catch ex As Exception
         MsgBox("Fehler beim Laden der Liste aller Datenbankbenutzer:" & vbCrLf & vbCrLf & _
            ex.Message, MsgBoxStyle.Exclamation, Application.ProductName & ": Fehler")
         Return Nothing
      Finally
         If fConnectionOpenedHere_CloseWhenExiting AndAlso DB_Connection.State <> ConnectionState.Closed Then DB_Connection.Close()
      End Try
   End Function

   ''' <summary>
   ''' Gibt einen einzelnen Datensatz zurück.
   ''' </summary>
   Public Function Load_Record(ByVal intRecord_PK As Integer) As DataRow
      Dim fConnectionOpenedHere_CloseWhenExiting As Boolean = (DB_Connection.State <> ConnectionState.Open)
      Try
         Dim dt As New DataTable(Me.ToString)
         Dim cmd As New SqlCommand("dbo.p_Employees_LoadRecord", DB_Connection)
         cmd.CommandType = CommandType.StoredProcedure
         cmd.Parameters.Add("@intEmployeeID", SqlDbType.NVarChar)
         cmd.Parameters("@intEmployeeID").Value = intRecord_PK

         If fConnectionOpenedHere_CloseWhenExiting AndAlso DB_Connection.State <> ConnectionState.Open Then DB_Connection.Open()
         Dim da As New SqlDataAdapter(cmd)
         da.Fill(dt)
         If dt.Rows.Count > 0 Then
            Return dt.Rows(0)
         Else
            Return Nothing
         End If

      Catch ex As Exception
         MsgBox("Fehler beim Laden eines Datenbankbenutzers:" & vbCrLf & vbCrLf & _
            ex.Message, MsgBoxStyle.Exclamation, Application.ProductName & ": Fehler")
         Return Nothing
      Finally
         If fConnectionOpenedHere_CloseWhenExiting AndAlso DB_Connection.State <> ConnectionState.Closed Then DB_Connection.Close()
      End Try
   End Function

   ''' <summary>
   ''' Speichert die Daten eines Benutzers.
   ''' </summary>
   Public Function Save_Record( _
            ByRef intRecord_PK As Integer, _
            ByVal strLastName As String, _
            ByVal strFirstName As String, _
            ByVal strTitle As String _
         ) As Boolean
      Dim fConnectionOpenedHere_CloseWhenExiting As Boolean = (DB_Connection.State <> ConnectionState.Open)
      Try
         Dim fSuccess As Boolean = False
         Dim cmd As New SqlCommand("dbo.p_Employees_SaveRecord", DB_Connection)
         cmd.CommandType = CommandType.StoredProcedure
         cmd.Parameters.Add("@intEmployeeID", SqlDbType.Int)
         cmd.Parameters.Add("@nvcLastName", SqlDbType.NVarChar)
         cmd.Parameters.Add("@nvcFirstName", SqlDbType.NVarChar)
         cmd.Parameters.Add("@nvcTitle", SqlDbType.NVarChar)

         cmd.Parameters("@intEmployeeID").Value = intRecord_PK
         cmd.Parameters("@nvcLastName").Value = strLastName
         cmd.Parameters("@nvcFirstName").Value = strLastName
         cmd.Parameters("@nvcTitle").Value = strLastName

         If fConnectionOpenedHere_CloseWhenExiting AndAlso DB_Connection.State <> ConnectionState.Open Then DB_Connection.Open()
         Dim intID_Returned As Integer = CType(cmd.ExecuteScalar, Integer)
         'fSuccess = (cmd.ExecuteNonQuery() > 0)
         fSuccess = (intID_Returned > 0)
         If fSuccess Then
            Dim cLH As New cSQLS_LockHandler(Me.TableName)
            cLH.Remove_Lock(intRecord_PK:=intRecord_PK, fShowMessageIfFailed:=False)
            intRecord_PK = intID_Returned
         End If
         Return True

      Catch ex As Exception
         MsgBox("Fehler beim Speichern eines Datenbankbenutzers:" & vbCrLf & vbCrLf & _
            ex.Message, MsgBoxStyle.Exclamation, Application.ProductName & ": Fehler")
         Return False
      Finally
         If fConnectionOpenedHere_CloseWhenExiting AndAlso DB_Connection.State <> ConnectionState.Closed Then DB_Connection.Close()
      End Try
   End Function

   ''' <summary>
   ''' Versucht, den übergebenen Benutzer aus der Datenbank zu löschen.
   ''' </summary>
   ''' <returns>True: DS wurde gelöscht; False: DS konnte nicht gelöscht werden, evt. aufgrund eines bestehenden Locks.</returns>
   ''' <remarks></remarks>
   Public Function Delete_Record(ByVal intRecord_PK As Integer) As Boolean
      Dim dr As SqlDataReader = Nothing
      Dim fConnectionOpenedHere_CloseWhenExiting As Boolean = (DB_Connection.State <> ConnectionState.Open)
      Try
         Dim cmd As New SqlCommand("dbo.p_Employees_DeleteRecord", DB_Connection)
         cmd.CommandType = CommandType.StoredProcedure
         cmd.Parameters.Add("@intEmployeeID", SqlDbType.NVarChar)
         cmd.Parameters("@intEmployeeID").Value = intRecord_PK

         If fConnectionOpenedHere_CloseWhenExiting AndAlso DB_Connection.State <> ConnectionState.Open Then DB_Connection.Open()
         dr = cmd.ExecuteReader
         If dr.HasRows Then
            dr.Read()
            'Das Feld "RecordWasDeleted" zeigt an, ob der DS gelöscht werden konnte, oder nicht.
            If CType(dr("RecordWasDeleted"), Boolean) Then
               'OK - DS wurde gelöscht.
               Return True
            Else
               'DS konnte nicht gelöscht werden, höchstwahrscheinlich, weil der DS von jemandem für die Bearbeitung gesperrt wurde.
               If IsDate(dr("Lock_DateTime")) Then
                  MsgBox("Der Datensatz konnte nicht gelöscht werden, weil er von Benutzer  " & dr("Lock_User").ToString & _
                     " am/um " & CType(dr("Lock_DateTime"), Date).ToString("dd.MM.yyyy\, HH:mm:ss") & _
                     " gesperrt wurde." & vbCrLf & _
                     "Bitte versuchen Sie es später oder kontaktieren Sie den genannten Benutzer.", _
                     MsgBoxStyle.Exclamation, Application.ProductName & ": Löschen nicht möglich")
               ElseIf Not CType(dr("RecordFound"), Boolean) Then
                  MsgBox("Der Datensatz konnte nicht gefunden werden." & vbCrLf & _
                     "Möglicherweise wurde der Datensatz durch einen anderen Benutzer gelöscht.", _
                     MsgBoxStyle.Exclamation, Application.ProductName & ": Löschen fehlgeschlagen")
               End If
               Return False
            End If
         Else
            'Keine Rückmeldung durch SP - sollte nicht passieren
            MsgBox("Der Datensatz konnte aufgrund eines Problems mit dem Datenbank Server nicht gelöscht werden." & vbCrLf & vbCrLf & _
               "Sollten Sie diese Meldung wiederholt erhalten, wenden Sie sich bitte an den Datenbank-Administrator!", _
               MsgBoxStyle.Critical, Application.ProductName & ": Löschen fehlgeschlagen")
         End If

      Catch exSQLS As SqlException
         If exSQLS.Number = 547 Then
            'RI-Fehler
            MsgBox("Der Datensatz konnte nicht gelöscht werden, da Verbindungen zu anderen Daten das Löschen verhindern " & _
               vbCrLf & vbCrLf & "Aktion abgebrochen.", _
               MsgBoxStyle.Exclamation, Application.ProductName & ": Löschen nicht möglich")
         Else
            MsgBox("Beim Löschen des Datensatzes trat folgender Datenbank-Fehler auf:" & vbCrLf & vbCrLf & _
               exSQLS.Message, MsgBoxStyle.Exclamation, Application.ProductName & ": Fehler")
         End If
      Catch ex As Exception
         MsgBox("Beim Löschen des Datensatzes trat folgender Fehler auf:" & vbCrLf & vbCrLf & _
            ex.Message, MsgBoxStyle.Exclamation, Application.ProductName & ": Fehler")
         Return False
      Finally
         If Not dr Is Nothing AndAlso Not dr.IsClosed Then dr.Close()
         If fConnectionOpenedHere_CloseWhenExiting AndAlso DB_Connection.State <> ConnectionState.Closed Then DB_Connection.Close()
      End Try
   End Function

   ''' <summary>
   ''' Will attempt to create a lock for the given record.
   ''' </summary>
   ''' <returns>True: Lock was created (record may be edited). False: A lock already existed, hence editing is to be prevented.</returns>
   Public Overridable Function Edit_Record(ByVal cLH As cSQLS_LockHandler, ByVal intRecord_PK As Integer) As Boolean
      Return cLH.Create_Lock(intRecord_PK)
   End Function

   ''' <summary>
   ''' Will remove the lock that has been created while editing the given record.
   ''' </summary>
   ''' <returns>True: Lock removed successfully or doesn't exist. False: Lock could not be removed (Error).</returns>
   Public Overridable Function Cancel_Edit(ByVal clh As cSQLS_LockHandler, ByVal intRecord_PK As Integer) As Boolean
      Return clh.Remove_Lock(intRecord_PK, False)
   End Function

   Public Sub New()
      MyBase.New()
   End Sub

   Protected Overrides Sub Finalize()
      MyBase.Finalize()
   End Sub

End Class

#End Region
