﻿Option Explicit On
Option Strict On

''' <summary>
''' Allgemeine Funktionen für die Verwendung in SQL-Server Datenbanken.
''' </summary>
Public Class cSQLS_Generics

   ''' <summary>
   ''' NZ-Funktion für Rückgabe von Strings
   ''' </summary>
   Public Shared Function nz(ByVal obj As Object, ByVal strReturnIfNull As String) As String
      If System.DBNull.Value.Equals(obj) Then
         Return strReturnIfNull
      Else
         Return obj.ToString
      End If
   End Function

   ''' <summary>
   ''' NZ-Funktion für Rückgabe von Integern
   ''' </summary>
   Public Shared Function nz(ByVal obj As Object, ByVal intReturnIfNull As Integer) As Integer
      If System.DBNull.Value.Equals(obj) Then
         Return intReturnIfNull
      Else
         Return CType(obj, Integer)
      End If
   End Function

   ''' <summary>
   ''' Textinhalte in SQL-kompatibles Format umwandeln (z.B. für direkte INSERTs)
   ''' </summary>
   ''' <param name="strValue">Umzuwandelnder Wert (String)</param>
   ''' <param name="fUnicode">True für Unicode/nvarchar und Konsorten (Prefix = N'...'), ansonsten False (varchar und Konsorten</param>
   ''' <param name="fPreceedingAsterisk">für LIKE-Suche / führendes "%"</param>
   ''' <param name="fTrailingAsterisk">für LIKE-Suche / anhängendes "%"</param>
   ''' <param name="strLIKE_Asterisk">Zeichen für LIKE-Suche ("%" für SQLS, "*" für JET)</param>
   ''' <param name="strQuoteCharacter">Zu verwendender Textbegrenzer (' für SQLS)</param>
   ''' <returns>Esape'ter String zur direkten Verwendung in SQL</returns>
   Shared Function ScrubInput( _
                  ByVal strValue As String, _
                  Optional ByVal fUnicode As Boolean = False, _
                  Optional ByVal fPreceedingAsterisk As Boolean = False, _
                  Optional ByVal fTrailingAsterisk As Boolean = False, _
                  Optional ByVal strLIKE_Asterisk As String = "%", _
                  Optional ByVal strQuoteCharacter As String = "'") As String
      If Trim(strValue) = String.Empty Then
         ScrubInput = "NULL"
         Exit Function
      End If

      ' Add asterisk for LIKE-lookups if requested
      If fPreceedingAsterisk Then strValue = strLIKE_Asterisk & strValue
      If fTrailingAsterisk Then strValue = strValue & strLIKE_Asterisk

      ' Escape quote-character.
      If strQuoteCharacter > String.Empty Then
         strValue = Replace(strValue, strQuoteCharacter, strQuoteCharacter & strQuoteCharacter)
      End If

      strValue = strQuoteCharacter & Trim(strValue) & strQuoteCharacter
      If fUnicode Then strValue = "N" & strValue

      Return strValue
   End Function

   ''' <summary>
   ''' Datumswert in SQL-kompatibles Format umwandeln (z.B. für direkte INSERTs)
   ''' </summary>
   ''' <param name="varValue">Datumswert (oder z.B. DBNULL oder "")</param>
   ''' <param name="strFormat">Zurückzugebenes Format</param>
   ''' <returns>Esape'ter String zur direkten Verwendung in SQL</returns>
   Shared Function ScrubDateTime( _
                  ByVal varValue As Object, _
                  Optional ByVal strFormat As String = "MM\/dd\/yyyy HH\:mm\:ss") As String
      If Not IsDate(varValue) Then
         Return "Null"
      Else
         Return "#" & Format(CDate(varValue), strFormat) & "#"
      End If
   End Function

End Class
