Option Explicit On
Option Strict On

'***** VERSION DATE: 2008-02-09 *****

Imports System.data.OleDb
Imports System.runtime.InteropServices
Imports Microsoft.VisualBasic

Public Class cGeneral

   'Wrapper for <">
   Public Const Q As String = """"

#Region " --- Language-dependent constants for DataInvalidInfo --- "

   '#Const LANGUAGE = "English"
#Const LANGUAGE = "German"

#If LANGUAGE = "English" Then
   Private Const _conLANG_DataInvalid_SaveQ As String = "Would you like to save this record anyway?"
   Private Const _conLANG_DataInvalid_SaveQ_Title As String = "Attention: data insufficient!"
   Private Const _conLANG_DataInvalid_Invalid_Title As String = "Error: data invalid!"

   Private Const _conLANG_AddToTable_Invalid As String = "Insufficient data, please check."
   Private Const _conLANG_AddToTable_Invalid_Title As String = "Error: data invalid!"

   Private Const _conLANG_GeneralErr As String = "Error %ErrorNumber%: " & vbCrLf & "Description: %ErrorDescription%"
   Private Const _conLANG_GeneralErr_Title As String = "Unexpected Error"
#ElseIf LANGUAGE = "German" Then
   Private Const _conLANG_DataInvalid_SaveQ As String = "Mchten Sie diesen Datensatz trotzdem speichern?"
   Private Const _conLANG_DataInvalid_SaveQ_Title As String = "Achtung: Daten unvollstndig"
   Private Const _conLANG_DataInvalid_Invalid_Title As String = "Fehler: Daten fehlerhaft"

   Private Const _conLANG_AddToTable_Invalid As String = "Unzureichende Daten - bitte prfen Sie Ihre Angaben."
   Private Const _conLANG_AddToTable_Invalid_Title As String = "Fehler: Daten fehlerhaft"

   Private Const _conLANG_GeneralErr As String = "Fehler %ErrorNumber%: " & vbCrLf & "Beschreibung: %ErrorDescription%"
   Private Const _conLANG_GeneralErr_Title As String = "Unerwarteter Fehler"
#End If

#End Region

   'Equivalent to the Access-DLookup-function
   Public Shared Function TLookup(ByVal strConnectionString As String, _
                                   ByVal strField As String, _
                                   ByVal strTable As String) As Object
      Return QueryData(strConnectionString, "SELECT [" & strField & "] FROM [" & strTable & "]")
   End Function
   Public Shared Function TLookup(ByVal strConnectionString As String, _
                                   ByVal strField As String, _
                                   ByVal strTable As String, _
                                   ByVal strWHERE As String) As Object
      Return QueryData(strConnectionString, "SELECT [" & strField & "] FROM [" & strTable & "] WHERE " & strWHERE)
   End Function

   'queries a single (!) value from a database
   Public Shared Function QueryData(ByVal strConnectionString As String, _
                             ByVal strSQL As String) As Object
      Dim cn As New OleDbConnection(strConnectionString)
      Dim cmd As New OleDbCommand(strSQL, cn)
      Dim obj As Object

      'open the connection, get the data, and close the connection
      cn.Open()
      obj = cmd.ExecuteScalar()
      cn.Close()
      Return obj
   End Function
   Public Shared Function QueryData(ByVal cn As OleDbConnection, _
                             ByVal strSQL As String) As Object
      Dim cmd As New OleDbCommand(strSQL, cn)
      Dim obj As Object
      Dim fConnectionOpenedHere_CloseWhenExiting As Boolean = cn.State <> ConnectionState.Open

      'open the connection, get the data, and close the connection
      If fConnectionOpenedHere_CloseWhenExiting Then cn.Open()
      obj = cmd.ExecuteScalar()
      If fConnectionOpenedHere_CloseWhenExiting Then cn.Close()
      Return obj
   End Function

   ''' <summary>
   ''' Adds a trailing backslash to the passed string if none is there.
   ''' </summary>
   Public Shared Function getBackslashedPath(ByVal str As String) As String
      If str.Substring(str.Length - 1, 1) <> "\" Then
         Return str & "\"
      Else
         Return str
      End If
   End Function

   ''' <summary>
   ''' Shows a user message that indicates that a record's underlying data is invalid.
   ''' </summary>
   ''' <param name="strMsg">The message to be presented to the user</param>
   ''' <param name="ctl">The control to set the focus to.</param>
   ''' <param name="fAllowContinuing">Allows the user to override the message and store/save the regularly invalid record. 
   ''' This function's return-value is meaningful only if this parameter has been set to True.</param>
   ''' <returns>True: Although the record is supposed to be invalid, the user chose to continue saving; False: do not save data / cancel the operation.</returns>
   ''' <remarks></remarks>
   Public Shared Function DataInvalidInfo( _
            ByVal strMsg As String, _
            ByVal ctl As Control, _
            Optional ByVal fAllowContinuing As Boolean = False) As Boolean
      Dim fContinue As Boolean = False
      If fAllowContinuing Then
         fContinue = (MsgBox(strMsg & vbCrLf & vbCrLf & _conLANG_DataInvalid_SaveQ, _
            MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, _conLANG_DataInvalid_SaveQ_Title) = vbYes)
      Else
         MsgBox(strMsg, MsgBoxStyle.Exclamation Or vbOKOnly, _conLANG_DataInvalid_Invalid_Title)
      End If

      Try
         'Try to set the focus to the control
         ctl.Focus()
         'If the control is a combobox, open its dropdown-list
         If TypeOf (ctl) Is ComboBox Then CType(ctl, ComboBox).DroppedDown = True
      Catch
         'Disregard any potential error (i.e. due to a disabled control)
      End Try
      Return fContinue
   End Function

   Public Shared Function GetUserName() As String
      If TypeOf My.User.CurrentPrincipal Is Security.Principal.WindowsPrincipal Then
         ' The application is using Windows authentication.
         ' The name format is DOMAIN\USERNAME.
         Dim parts() As String = Split(My.User.Name, "\")
         Dim username As String = parts(1)
         Return username
      Else
         ' The application is using custom authentication.
         Return My.User.Name
      End If
   End Function

   ''' <summary>
   ''' Returns the version-number information of the current application.
   ''' </summary>
   ''' <returns>Version number as Major.Minor.Build.Private</returns>
   ''' <remarks></remarks>
   Public Shared Function GetVersion() As String
      With System.Diagnostics.FileVersionInfo.GetVersionInfo( _
        System.Reflection.Assembly.GetExecutingAssembly.Location)
         Return .FileMajorPart & "." & .FileMinorPart & "." & _
           .FileBuildPart & "." & .FilePrivatePart
      End With
   End Function

   ''' <summary>
   ''' Checks whether a given filename is valid within Windows-environments.
   ''' </summary>
   ''' <param name="strFilename_NoPath">Filename (without path-specs) to be validated</param>
   ''' <returns>True: Filename is valid; False: Filename is not valid</returns>
   ''' <remarks></remarks>
   Public Shared Function ValidateFilename(ByVal strFilename_NoPath As String) As Boolean
      Dim rex As New System.Text.RegularExpressions.Regex("^[^\\\./:\*\?\""<>\|]{1}[^\\/:\*\?\""<>\|]{0,254}$")
      Return (Not rex.IsMatch(strFilename_NoPath))
   End Function

   ''' <summary>
   ''' Splits a given string into several parts. Returns a List of with each part being a single 
   ''' item in the list.
   ''' </summary>
   ''' <param name="strText">The text to be split up.</param>
   ''' <param name="intMaxLen">The maximum length for each separate item in the list.</param>
   ''' <param name="intCountOfFields_Max">The maximum number of parts to create. If the length of strText exceeds {[intMaxLen] * [intCountOfFields_Max]},
   ''' it will be truncated.</param>
   ''' <param name="intThreshold_LastWord_MaxLengthBeforeSplitting">The method will attempt to split the string only between spaces, that is, words are
   ''' generally to be maintained (i.e. for search-operations in the combined contents). 
   ''' This variable thus specifies the maximum number of characters that a single word may contain at the end of a block before the word itself gets 
   ''' separated. This is to avoid that the overall maximum of combined field is exceeded, i.e. a combined contents of 500 characters to be split up
   ''' in two fields with 255 characters max., with only 200 chars in field #1 due to a very long word could result in field #2 containing 300 characters.
   ''' In this case, the max. length would need to be set to [5] as (500 - 255+255) / 2 = 5.</param>
   ''' <param name="fShowMessageIfTextWouldBeTruncated">If True (the default), the method will show a messagebox if the string gets truncated, i.e. if
   ''' there is characters remaining that do not fit into the string-blocks.</param>
   ''' <returns></returns>
   ''' <remarks></remarks>
   Public Shared Function SplitUpString( _
            ByVal strText As String, _
            ByVal intMaxLen As Integer, _
            ByVal intCountOfFields_Max As Integer, _
            ByVal intThreshold_LastWord_MaxLengthBeforeSplitting As Integer, _
            Optional ByVal fShowMessageIfTextWouldBeTruncated As Boolean = True _
         ) As List(Of String)
      Dim lstFields As New List(Of String)
      Dim sbTemp As New System.Text.StringBuilder(intCountOfFields_Max * intMaxLen)

      If strText.Length <= intMaxLen Then
         'No need to split - the contents of the string completely fits into the first element.
         'Create the requested number of elements with the text in the first element and return
         lstFields.Add(strText)
         For intCounter As Integer = 1 To (intCountOfFields_Max - 1)
            lstFields.Add("")
         Next
         Return lstFields
      Else
         'Split up the string
         sbTemp.Append(strText)

         'Split the string as long as there is characters left AND the max. amount of fields has not been exceeded.
         While sbTemp.Length > 0 AndAlso lstFields.Count < intCountOfFields_Max
            'Search the first space-character beginning at the position of the last character that may be inserted into field #1
            If sbTemp.Length > intMaxLen Then
               Dim intSpacePos As Integer = sbTemp.ToString.IndexOf(" ", intMaxLen - 1)
               If intSpacePos > intMaxLen Then
                  'Loop backwards through the string, finding the first possible occurance of a space-character, which is where
                  'the string may be split.
                  intSpacePos = (intMaxLen - 1)
                  Dim intCountOfCharactersTested As Integer = 0
                  While sbTemp.ToString.Substring(intSpacePos, 1) <> " " And intCountOfCharactersTested <= intThreshold_LastWord_MaxLengthBeforeSplitting
                     'Go back one character
                     intSpacePos -= 1
                     'Count the number of characters that have been tested (break operation if the maximum is exceeded)
                     intCountOfCharactersTested += 1
                  End While
                  If intCountOfCharactersTested > intThreshold_LastWord_MaxLengthBeforeSplitting Then
                     'If the max. number of characters for a single word has been exceeded, DO split the word
                     intSpacePos = sbTemp.ToString.IndexOf(" ", intMaxLen - 1)
                  Else
                     'Use the position that has been found
                     '(Do nothing here)
                  End If
               ElseIf intSpacePos < 0 Then
                  'There is no space in the text - split at the max. position
                  intSpacePos = intMaxLen
               Else
                  'The text may be split at the end of the current block (exact position match - the next block will start out with a space)
               End If
               lstFields.Add(sbTemp.ToString.Substring(0, intSpacePos))
               sbTemp.Remove(0, intSpacePos)
            Else
               'This is the last block of text - it fits in completely.
               lstFields.Add(sbTemp.ToString)
               sbTemp.Remove(0, sbTemp.Length)
            End If
         End While

         'Add empty fields if the max. count has not been met
         While lstFields.Count < intCountOfFields_Max
            lstFields.Add("")
         End While

         'If there is characters left in the StringBuilder, that text will be skipped, so show a message
         If fShowMessageIfTextWouldBeTruncated AndAlso sbTemp.Length > 0 Then
            MsgBox("The text you entered is too long. It will be truncated according to the maximum size that is allowed.", _
               MsgBoxStyle.Exclamation, Application.ProductName & ": Text too long!")
         End If

         Return lstFields
      End If
   End Function

   ''' <summary>
   ''' Returns whether the passed form is currently open/loaded.
   ''' </summary>
   ''' <param name="frm">The form to be tested.</param>
   ''' <returns>True if the form is open (i.e. contained in the My.Application.OpenForms collection), False otherwise.</returns>
   Public Shared Function IsOpen(ByVal frm As Form) As Boolean
      Dim fIsOpen As Boolean = False
      For Each frmCheck As Form In My.Application.OpenForms
         If ReferenceEquals(frmCheck, frm) Then
            fIsOpen = True
            Exit For
         End If
      Next
      Return fIsOpen
   End Function

   ''' <summary>
   ''' Returns an Integer that may serve as an indicator as to when to update a wait-message that is being shown during a lengthy operation.
   ''' The total number of rows passed here will influence how often the message is to be shown.
   ''' 
   ''' See the function-code for an example of how to use this function within an operation
   ''' </summary>
   ''' <param name="intTotalRowCount"></param>
   ''' <returns></returns>
   ''' <remarks></remarks>
   Public Shared Function WaitMessage_GetModOperator(ByVal intTotalRowCount As Integer) As Integer
      'Usage in calling process:
      'Dim dtDataTable as DataTable = ...
      'Dim intPwCounter_CurrentRow As Integer = 1
      'Dim intModOperator As Integer = cGeneral.WaitMessage_GetModOperator(dtDataTable.Rows.Count)
      ''For Each r As DataRow In dtDataTable.Rows
      '   If (intPwCounter_CurrentRow Mod intModOperator = 0) Then
      '      frmPW.SetMessage("Doing something (" & intPwCounter_CurrentRow.ToString & "/" & dtDataTable.Rows.Count.ToString & ") ...")
      '   End If
      '
      '   Do something ...
      '
      '   intPwCounter_CurrentRow += 1
      'Next r

      Dim intModOperator As Integer
      Select Case intTotalRowCount
         Case 0 To 5 : intModOperator = 1
         Case 6 To 50 : intModOperator = 5
         Case 51 To 100 : intModOperator = 10
         Case 101 To 500 : intModOperator = 20
         Case 501 To 1000 : intModOperator = 50
         Case Else : intModOperator = 100
      End Select
      Return intModOperator
   End Function

   ''' <summary>
   ''' Generic method to change a DataGridView's datasheet-color (background of records/items and, optionally, the selector's fore- and backcolor) 
   ''' according to the control's .Enabled property.
   ''' </summary>
   ''' <param name="sender">Reference to the control (as object), i.e. the reference as received by a control's EnabledChanged-event.</param>
   ''' <remarks></remarks>
   Public Shared Sub GridView_EnabledChanged(ByVal sender As Object)
      With DirectCast(sender, DataGridView)
         If .Enabled Then
            .BackgroundColor = SystemColors.AppWorkspace
            .DefaultCellStyle.BackColor = Color.White
            .DefaultCellStyle.ForeColor = Color.Black
            .ColumnHeadersDefaultCellStyle.BackColor = SystemColors.Control
            .ColumnHeadersDefaultCellStyle.ForeColor = Color.Black
            .DefaultCellStyle.SelectionBackColor = SystemColors.Highlight
            .DefaultCellStyle.SelectionForeColor = SystemColors.HighlightText
         Else
            .BackgroundColor = SystemColors.ControlLight
            .DefaultCellStyle.BackColor = SystemColors.ControlLight
            .DefaultCellStyle.ForeColor = SystemColors.GrayText
            .ColumnHeadersDefaultCellStyle.BackColor = SystemColors.ControlLight
            .ColumnHeadersDefaultCellStyle.ForeColor = SystemColors.GrayText
            'Use the system-color and adjust it so that the selection bar's appearance changes, i.e. looks disabled as well
            Dim cColorChanger As New cHLS_RGB(SystemColors.Highlight)
            cColorChanger.Saturation = 0.5
            cColorChanger.LightenColor(0.5)
            .DefaultCellStyle.SelectionBackColor = cColorChanger.Color
            cColorChanger = New cHLS_RGB(SystemColors.HighlightText)
            cColorChanger.Saturation = 0.1
            cColorChanger.Luminance = 0.3 '0.8
            .DefaultCellStyle.SelectionForeColor = cColorChanger.Color
         End If
      End With
   End Sub

End Class
