﻿Option Explicit On
Option Strict On

Public Class frm03b_Lock_ExpirationWarning

#Region " --- Member Variables --- "

   ''' <summary>
   ''' Holdes the date/time when the record will automatically be unlocked
   ''' and this form will be closed.
   ''' </summary>
   Private _dteAutoRemoveAt As Date

   ''' <summary>
   ''' Holdes the date/time when the form will be shown
   ''' </summary>
   Private _dteShowFormAt As Date

   ''' <summary>
   ''' Local reference to the LockHandler class (this class will actually 
   ''' create instances of this form and pass a reference to itself).
   ''' </summary>
   Private _cLH As cSQLS_LockHandler

   ''' <summary>
   ''' Local reference to the ID of the locked record
   ''' </summary>
   Private _intRecord_PK As Integer

   ''' <summary>
   ''' Prevents closing of this form by any other means than the buttons or the CloseFormWithoutAction() method.
   ''' </summary>
   Private _fAllowClosing As Boolean

#End Region

#Region " --- Public Methods --- "

   ''' <summary>
   ''' To be called from outside, i.e. to close the form without any lock-related action if the lock has been removed by any other mechanism
   ''' </summary>
   ''' <remarks></remarks>
   Public Sub CloseFormWithoutAction()
      tmrCountdown.Enabled = False
      _fAllowClosing = True
      Me.Close()
   End Sub

#End Region

#Region " --- Form-/Control-Events --- "

   Private Sub tmrCountdown_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrCountdown.Tick
      Dim intSecondsRemaining_BeforeRemovingLock As Integer = CType(DateDiff(DateInterval.Second, Date.Now, _dteAutoRemoveAt), Integer)

      '1. Check whether to show the form or not
      If Not Me.Visible Then
         If Date.Now >= _dteShowFormAt Then
            Me.Visible = True
            'Additionally, disable the form that produced the lock
            If Not _cLH.CallerForm Is Nothing Then
               _cLH.CallerForm.Enabled = False
            End If
            'From now on, check every second.
            tmrCountdown.Interval = 1000
         End If
      End If

      '2. Update the countdown-label
      Dim ts As New TimeSpan(0, 0, intSecondsRemaining_BeforeRemovingLock)
      Dim sbCountdownText As New System.Text.StringBuilder
      If ts.Hours > 0 Then sbCountdownText.Append(ts.Hours.ToString("00") & ":")
      sbCountdownText.Append(ts.Minutes.ToString("00") & ":")
      sbCountdownText.Append(ts.Seconds.ToString("00"))
      lblCountdown.Text = sbCountdownText.ToString
      Me.Refresh()

      '3. Check whether to automatically unlock
      If intSecondsRemaining_BeforeRemovingLock < 1 Then
         RemoveLock(True)
         Me.Close()
      End If
   End Sub

   Private Sub cmdLock_Renew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdLock_Renew.Click
      'Renew the lock
      tmrCountdown.Enabled = False
      _cLH.Create_Lock(_intRecord_PK)
      _fAllowClosing = True
      Me.Close()
   End Sub

   Private Sub cmdLock_Cancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdLock_Cancel.Click
      'Remove the lock
      RemoveLock(False)
      Me.Close()
   End Sub

   Private Sub frmLock_ExpirationWarning_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
      'Prevent the user from closing the form via anything but the two buttons 
      '(these will set _fAllowClosing before attempting to close the form).
      e.Cancel = (Not _fAllowClosing)

      If Not e.Cancel AndAlso _cLH.CallerForm IsNot Nothing Then
         _cLH.CallerForm.Enabled = True
      End If
   End Sub

#End Region

#Region " --- Constructor --- "

   Public Sub New( _
            ByRef cLH As cSQLS_LockHandler, _
            ByVal intRecord_PK As Integer)
      InitializeComponent()

      'Copy the reference to the lock-class for local usage
      _cLH = cLH

      'Calculate the time at which the lock is to automatically be removed
      _dteAutoRemoveAt = DateAdd(DateInterval.Second, _cLH.LockSettings_Duration_Seconds, Date.Now)
      'Calculate the time at which this form is to be shown
      _dteShowFormAt = DateAdd(DateInterval.Second, (_cLH.LockSettings_ShowMessageForm_BeforeLockExpires_Seconds * -1), _dteAutoRemoveAt) ' dteAutoUnlockAt

      'Store the ID of the locked record
      _intRecord_PK = intRecord_PK

      'Initialize the countdown-label
      lblCountdown.Text = DateDiff(DateInterval.Second, _dteAutoRemoveAt, Date.Now).ToString("hh:mm:ss")

      'The first time the timer needs to fire would really be at the exact point in time when the local date/time corresponds to _dteShowFormAt.
      'However, just to be safe, we'll start the timer a minute before that time.
      If _cLH.LockSettings_Duration_Seconds > 60 Then
         tmrCountdown.Interval = (_cLH.LockSettings_Duration_Seconds - 60) * 1000
      Else
         'Pretty short lock-time, huh! Alright, then the timer should immediately start 
         'in 1s intervals  (i.e. when debugging, reaction times may be very short).
         tmrCountdown.Interval = 1000
      End If
      tmrCountdown.Enabled = True
   End Sub

#End Region

#Region " --- Helpers --- "

   Public Sub RemoveLock(ByVal fAutoRemoval As Boolean)
      tmrCountdown.Enabled = False
      _cLH.Remove_Lock(_intRecord_PK, False)
      _fAllowClosing = True
   End Sub

#End Region

End Class