﻿Option Explicit On
Option Strict On

Public Class frm03d_Employees_Inherited
   Inherits frm03a_BASE_DataForm

#Region " --- Stuff specific for this form --- "

#Region " --- Members --- "

   'Data-classes
   Private _DataClass As cSQLS_Employees

   'Columns in the Records grid
   Private Const _conGridCol_Record_ID As Integer = 0
   Private Const _conGridCol_Record_Descriptor As Integer = 1
   Private Const _conGridCol_Record_Notes As Integer = 2

   Private _strRecordDescriptor As String = "Employee"
   Private _strFormTitle As String = "Employees"
   Private _strRecordDesignator_Singular As String = "Employee"
   Private _strRecordDesignator_Plural As String = "Employees"

#End Region

#Region " --- Properties --- "

   ''' <summary>
   ''' Mechanism to transport the data-class between this form and
   ''' the parent/base.
   ''' </summary>
   ''' <value></value>
   ''' <returns></returns>
   ''' <remarks></remarks>
   Friend ReadOnly Property DataClass() As cSQLS_Employees
      Get
         If _DataClass Is Nothing Then
            _DataClass = New cSQLS_Employees
         End If
         Return _DataClass
      End Get
   End Property

#End Region

#End Region

#Region " --- Properties defined in the DataForm base class that need to be overrided here --- "

   Friend Overrides ReadOnly Property RecordDesignator_Singular() As String
      Get
         Return _strRecordDesignator_Singular
      End Get
   End Property

   Friend Overrides ReadOnly Property RecordDesignator_Plural() As String
      Get
         Return _strRecordDesignator_Plural
      End Get
   End Property

   Friend Overrides ReadOnly Property GridControl() As DataGridView
      Get
         Return dgRecords
      End Get
   End Property

   Friend Overrides ReadOnly Property Grid_ColumnIndex_Record_ID() As Integer
      Get
         Return _conGridCol_Record_ID
      End Get
   End Property

#End Region

#Region " --- Methods defined in the DataForm base class that need to be overrided here--- "

   Friend Overrides Property LockHandler() As cSQLS_LockHandler
      Get
         If _cLH Is Nothing Then
            _cLH = New cSQLS_LockHandler(Me.DataClass.TableName)
         End If
         Return _cLH
      End Get
      Set(ByVal value As cSQLS_LockHandler)
         Throw New NotImplementedException("LockHandler-Property (Set) has not been provided by the inheritor.")
         _cLH = value
      End Set
   End Property

   Friend Overrides Sub DataControls_SetLockstate(ByVal fLockControls As Boolean)
      'Dis-/enable editing in data-controls
      txtEmployeeID.ReadOnly = fLockControls
      txtLastName.ReadOnly = fLockControls
      txtFirstName.ReadOnly = fLockControls
      txtTitle.ReadOnly = fLockControls
   End Sub

   Friend Overrides Sub EmptyControls()
      txtEmployeeID.Text = String.Empty
      txtLastName.Text = String.Empty
      txtFirstName.Text = String.Empty
      txtTitle.Text = String.Empty
      Me.Dirty = False
   End Sub

   Friend Overrides Sub FormData_EditModeCanceling_RemoveLock()
      'Locks are only being used with SQLS, so skip this method if we're using JET
      DataClass.Cancel_Edit(Me.LockHandler, Me.Record_ID)
   End Sub

   Friend Overrides Function FormData_EditModeEntering_LockCreated() As Boolean
      Return DataClass.Edit_Record(Me.LockHandler, Record_ID)
   End Function

   Friend Overrides Sub FormData_LoadRecord(ByVal Record_ID As Integer)
      Dim r As DataRow = Nothing
      r = DataClass.Load_Record(Record_ID)

      If r Is Nothing Then
         'None found! (Should only happen if the selected record has been flagged as "deleted" by another user)
         MsgBox("Error: the selected " & RecordDesignator_Singular & " could not be located in the database." & vbCrLf & vbCrLf & _
            "It may be possible that this record has been deleted by someone else." & vbCrLf & _
            "Click OK to continue.", _
            MsgBoxStyle.Exclamation, Application.ProductName & ": Error")
         Record_ID = Nothing
         ReloadGrid()
      Else
         'Load data into controls
         txtEmployeeID.Text = r(DataClass.FieldName(cSQLS_Employees.eFields.EmployeeID)).ToString
         txtLastName.Text = r(DataClass.FieldName(cSQLS_Employees.eFields.LastName)).ToString
         txtFirstName.Text = r(DataClass.FieldName(cSQLS_Employees.eFields.FirstName)).ToString
         txtTitle.Text = r(DataClass.FieldName(cSQLS_Employees.eFields.Title)).ToString
         tsslCreator.Text = "" 'Wird hier nicht benutzt! DataClass.GenericFields_CreateUser(r)
         tsslUpdater.Text = "" 'Wird hier nicht benutzt! DataClass.GenericFields_UpdateUser(r)

         'Store the Create- and Update-user
         Me.CreateUser = "" 'Wird hier nicht benutzt! 'r(DataClass.GenericFieldName(ILogBook_DataAccess_GenericFields.eGenericFieldnames.CreatedBy)).ToString
         Me.UpdateUser = "" 'Wird hier nicht benutzt! 'r(DataClass.GenericFieldName(ILogBook_DataAccess_GenericFields.eGenericFieldnames.UpdatedBy)).ToString

         Me.Dirty = False
      End If
   End Sub

   Friend Overrides Function FormData_Save(Optional ByRef fShowMessage As Boolean = True) As Boolean
      Dim fSaveSuccess As Boolean = _
         DataClass.Save_Record( _
            Record_ID, _
            txtLastName.Text, _
            txtFirstName.Text, _
            txtTitle.Text _
            )
      Return fSaveSuccess
   End Function

   Friend Overrides Function IsDataValid() As Boolean
      'If txtEmployeeID.Text = String.Empty Then cGeneral.DataInvalidInfo("No information for this " & RecordDesignator_Singular & " stated - record cannot be saved.", txtEmployeeID) : Return False

      If txtLastName.Text = String.Empty Then cGeneral.DataInvalidInfo("Please enter the last name for this " & RecordDesignator_Singular & ".", txtLastName) : Return False
      If txtFirstName.Text = String.Empty Then cGeneral.DataInvalidInfo("Please enter the first name for this " & RecordDesignator_Singular & ".", txtFirstName) : Return False
      If txtTitle.Text = String.Empty Then cGeneral.DataInvalidInfo("Please enter the " & RecordDesignator_Singular & "'s title.", txtTitle) : Return False

      'Got here? Then data is valid.
      Return True
   End Function

   Friend Overrides Function RecordList() As DataTable
      Return DataClass.Records_DataTable
   End Function

   Friend Overrides Sub FormCaption(Optional ByVal strAppendText As String = "")
      Me.Text = _strFormTitle
      If strAppendText <> String.Empty Then
         Me.Text &= ": " & strAppendText
      End If
   End Sub

#End Region

#Region " --- Events being raised by the DataForm base class --- "

   Private Sub Form_Record_BeforeDelete(ByVal Record_ID__ToBeDeleted As Integer, ByRef Cancel As Boolean) Handles Me.Record_BeforeDelete
      Cancel = (Not cUserProps.Permissions_DeletionAllowed(Me.CreateUser))
   End Sub

   Private Sub Form_Record_Delete(ByVal Record_ID__ToBeDeleted As Integer) Handles Me.Record_Delete
      'The GUID of the record that is to be selected instead of the one that is to be deleted
      Dim Record_ID_SelectAfterDelete As Integer = CType(Grid_ColumnValue_SelectedRecord_PreviousOrNext(dgRecords, _conGridCol_Record_ID), Integer)
      If DataClass.Delete_Record(Record_ID__ToBeDeleted) Then
         MsgBox("The record has been deleted.", MsgBoxStyle.Information, Application.ProductName & ": Record deleted")
         Me.Record_ID = Record_ID_SelectAfterDelete
      End If
      ReloadGrid()
   End Sub

   Private Sub Form_DirtyState_BeforeChange() Handles Me.DirtyState_BeforeChange
   End Sub

   Private Sub Form_DirtyStateChanged(ByVal NewState As Boolean) Handles Me.DirtyStateChanged
   End Sub

   Private Sub Form_EditState_LockAutoRemoved(ByVal TableName As String, ByVal RecordID As Integer, ByVal LockAutoRemoved As Boolean) Handles Me.EditState_LockAutoRemoved
      If Me.EditStateActive Then
         cmdCancelEdit_Click(Nothing, Nothing)
         If LockAutoRemoved Then
            'Dem Benutzer anzeigen, dass seine Änderungen aufgrund der Zeitüberschreitung automatisch verworfen wurden
            MsgBox("The changes that you applied to the currently selected have been undone due to a lock-timeout" & vbCrLf & _
               "(the record was locked too long)." & vbCrLf & vbCrLf & _
               "Please apply your changes again, if required.", _
               MsgBoxStyle.Exclamation, Application.ProductName)
         End If
      End If
   End Sub

   Private Sub frmAdminList_FormLoading() Handles Me.FormLoading
      'Bind the grid-columns (may be done in the designer instead)
      'colEmployees_EmployeeID.DataPropertyName = DataClass.FieldName(cSQLS_Employees.eFields. ...)
      'col...

      'Focus the Grid-control
      GridControl.Focus()
   End Sub

   ''' <summary>
   ''' All controls that are to trigger the form's Dirty-property must be added here
   ''' </summary>
   ''' <param name="sender"></param>
   ''' <param name="e"></param>
   ''' <remarks></remarks>
   Private Sub Form_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
      'If Not cUserProps.Permissions_IsMemberOf(cSQLS_Permissions_Administration.ePermissionGroups.Admins, True, True) Then
      '   Me.Close()
      '   Return
      'End If

      'ToolTip.Active = My.Settings.ShowTooltips
      'HelpProvider.HelpNamespace = cHelpAccess.HelpFile_FullFilename

      'Add Dirty-handlers to all data-controls
      AddHandler txtEmployeeID.TextChanged, AddressOf Form_Dirty
      AddHandler txtLastName.TextChanged, AddressOf Form_Dirty
      AddHandler txtFirstName.TextChanged, AddressOf Form_Dirty
      AddHandler txtTitle.TextChanged, AddressOf Form_Dirty
   End Sub

   Private Sub Form_NewRecord_StateChanged(ByVal NewState As Boolean) Handles Me.NewRecord_StateChanged
   End Sub

   Private Sub Form_RecordSaved() Handles Me.RecordSaved
   End Sub

#End Region

End Class