USE NORTHWIND
GO

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[tblRecord_Locks]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	DROP TABLE [dbo].[tblRecord_Locks]
GO
IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[tblRecord_Locks_Settings]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	DROP TABLE [dbo].[tblRecord_Locks_Settings]
GO
IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[p_Lock_Attempt_Create]') AND OBJECTPROPERTY(id,N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[p_Lock_Attempt_Create]
GO
IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[p_Lock_Attempt_Remove]') AND OBJECTPROPERTY(id,N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[p_Lock_Attempt_Remove]
GO
IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[p_Lock_Attempt_Remove_ByLockGuid]') AND OBJECTPROPERTY(id,N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[p_Lock_Attempt_Remove_ByLockGuid]
GO
IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[p_Employees_LoadRecord]') AND OBJECTPROPERTY(id,N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[p_Employees_LoadRecord]
GO
IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[p_Employees_RecordList]') AND OBJECTPROPERTY(id,N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[p_Employees_RecordList]
GO
IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[p_Employees_SaveRecord]') AND OBJECTPROPERTY(id,N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[p_Employees_SaveRecord]
GO
IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[p_Employees_DeleteRecord]') AND OBJECTPROPERTY(id,N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[p_Employees_DeleteRecord]
GO

CREATE TABLE [dbo].[tblRecord_Locks](
	[Lock_GUID] [uniqueidentifier] ROWGUIDCOL  NOT NULL CONSTRAINT [DF_tblRecord_Locks_Lock_GUID]  DEFAULT (newid()),
	[Lock_DateTime] [datetime] NOT NULL,
	[Lock_User] [nvarchar](255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Lock_Table] [nvarchar](150) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL,
	[Lock_Record_PK] [int] NOT NULL,
 CONSTRAINT [PK_RL] PRIMARY KEY CLUSTERED 
(
	[Lock_GUID] ASC
) ON [PRIMARY]
) ON [PRIMARY]

CREATE TABLE [dbo].[tblRecord_Locks_Settings](
	[LS_GUID] [uniqueidentifier] ROWGUIDCOL  NOT NULL CONSTRAINT [DF_tblRecord_Locks_Settings_Lock_GUID]  DEFAULT (newid()),
	[LS_MaxLockTime_Seconds] [int] NOT NULL,
	[LS_ShowForm_SecondsBeforeExpiring] [int] NOT NULL,
	[LS_SP_Unlock_Threshold] [int] NOT NULL,
 CONSTRAINT [PK_RLS] PRIMARY KEY CLUSTERED 
(
	[LS_GUID] ASC
) ON [PRIMARY]
) ON [PRIMARY]
GO

-- Settings fr den Lock-Handler einfgen
INSERT INTO dbo.tblRecord_Locks_Settings
           ( LS_MaxLockTime_Seconds, LS_ShowForm_SecondsBeforeExpiring, LS_SP_Unlock_Threshold )
     VALUES ( 20, 15, 30 )
GO

/*
	Attempts to lock the record identified by @intRecord_PK in the table called [@vcTableName].

	Information is being returned as a single record:
		Lock_User (varchar)			-	if a lock already exists, this field will contain the
												windows-logon of the user that already locked the record.
												If the lock was successfully created, the user that was
												passed in @nvcUser_WinLogon will be contained.
		Lock_Datetime (datetime)	-	if a lock already exists, this field will contain the timestamp
												indicating when the lock was originally created.
												If the lock was successfully created, the date/time when the
												lock was created will be contained.
		Lock_Created (bit)			-	if a lock already exists, this field will contain 0/FALSE
												(indicating that the lock could not be created) or
												otherwise 1/TRUE (indicating the lock has been created).

*/
CREATE PROCEDURE [dbo].[p_Lock_Attempt_Create]
(
	@nvcTableName nvarchar(255),					-- Name of the table that contains the record that is to be locked.
	@nvcUser_WinLogon nvarchar(255),				-- Windows-logon name of the locking user
	@intRecord_PK int									-- PK of the record that is being locked (as int) 
)
AS
BEGIN
	DECLARE @nvcExistingLock_User_WinLogon nvarchar(255)
	DECLARE @dteLock_Created datetime
	DECLARE @bitLock_Exists bit
	DECLARE @intLock_MaxSeconds int

	-- Read the lock-settings
	-- If a lock is existent but older than the amount of seconds calculated here, it will be removed here.
	SELECT @intLock_MaxSeconds = (LS_MaxLockTime_Seconds + LS_SP_Unlock_Threshold)
	FROM dbo.tblRecord_Locks_Settings

	-- Start with removing all locks that are "too old" (current table only).
	DELETE 
	FROM dbo.tblRecord_Locks
	WHERE
			( Lock_Table = @nvcTableName )
		AND
			( DATEDIFF(second, Lock_DateTime, GETDATE()) > @intLock_MaxSeconds )
	
	-- Check as to whether the given record has already been locked
	SELECT 
		@bitLock_Exists = CASE WHEN Lock_DateTime IS NULL THEN 0 ELSE 1 END,
		@dteLock_Created = Lock_DateTime,
		@nvcExistingLock_User_WinLogon = Lock_User
	FROM dbo.tblRecord_Locks
	WHERE
			( Lock_Table = @nvcTableName )
		AND
			( Lock_Record_PK = @intRecord_PK )

	IF @bitLock_Exists = 1 BEGIN
		-- A lock already exists
		IF @nvcExistingLock_User_WinLogon = @nvcUser_WinLogon BEGIN
			-- The user attempting to lock and the "owner" of the lock are the same, so update the lock's timestamp
			SET @dteLock_Created = GETDATE()
			
			UPDATE dbo.tblRecord_Locks
			SET Lock_DateTime = @dteLock_Created 
			WHERE
					( Lock_Table = @nvcTableName )
				AND
					( Lock_Record_PK = @intRecord_PK )

			-- Return info about the lock that was just updated
			SELECT 
				@nvcUser_WinLogon AS Lock_User,
				@dteLock_Created AS Lock_DateTime,
				1 AS Lock_Created
		END ELSE BEGIN
			-- The user attempting to lock and the "owner" of the lock are not the same, so just inform the caller and do not create a lock.
			SELECT 
				@nvcExistingLock_User_WinLogon AS Lock_User,
				@dteLock_Created AS Lock_DateTime,
				0 AS Lock_Created
		END
	END ELSE BEGIN
		-- A lock does not exist and may hence be created.
		SET @dteLock_Created = GETDATE()

		INSERT INTO dbo.tblRecord_Locks
			( Lock_DateTime, Lock_User, Lock_Table, Lock_Record_PK )
		VALUES (
			@dteLock_Created,
			@nvcUser_WinLogon,
			@nvcTableName,
			@intRecord_PK 		
			)
		-- Return info about the lock that was just created
		SELECT 
			@nvcUser_WinLogon AS Lock_User,
			@dteLock_Created AS Lock_DateTime,
			1 AS Lock_Created
	END
RETURN
END
GO

/*

	Attempts to remove a record-lock identified by @intRecord_PK in the table called [@vcTableName].
	A lock may only be removed by the user that originally created a lock OR a user that has
	administrative privileges (member of the "Admin"-group).

	Information is being returned as a single record:
		Lock_NotRemoved_Reason (nvarchar)-	If the lock could not be removed (Lock_Removed = 0), this field will 
														contain information information about the reason.
		Lock_Removed (bit)					-	If the lock was successfully removed, this field will contain 1/TRUE
														(indicating that the lock has been removed).
														Otherwise 0/FALSE will be returned in which case information about
														the reason will be contained in the PermissionDenied_Info-field.
		Lock_StillExists (bit)				-  This field will show as to whether, when returning from this SP,
														a lock still exists for the requested record.
														If i.e. the lock could not be removed due to the fact that there WAS no
														lock, Lock_Removed will show 0. In this case though, Lock_StillExists 
														will also show 0, indicating that there is no lock for that record.

*/
CREATE PROCEDURE [dbo].[p_Lock_Attempt_Remove]
(
	@nvcTableName nvarchar(255),			-- Name of the table that contains the record that is to be locked.
	@nvcUser_WinLogon nvarchar(255),		-- Windows-logon name of the locking user
	@intRecord_PK int							-- ID of the record that is being locked
)
AS
BEGIN
	DECLARE @nvcExistingLock_User_WinLogon nvarchar(255)
	DECLARE @dteLock_Created datetime
	DECLARE @bitLock_Exists bit
	DECLARE @bitLock_Removed bit
	DECLARE @bitPrivilegesSufficient bit
	DECLARE @nvcNotRemoved_Reason nvarchar(400)
	DECLARE @bitRemoveAllowed bit

	-- Default: No lock found
	SET @bitLock_Exists = 0

	-- Default: Lock could not be removed
	SET @bitLock_Removed = 0

	-- Default: The current user's privileges are not sufficient in order to remove a lock from someone else
	SET @bitPrivilegesSufficient = 0

	-- Default: no resp. empty reason-message
	SET @nvcNotRemoved_Reason = N''

	-- Default: do not delete the lock-record
	SET @bitRemoveAllowed = 0

	-- Check as to whether the given record has been locked at all
	SELECT 
		@bitLock_Exists = CASE WHEN Lock_DateTime IS NULL THEN 0 ELSE 1 END,
		@dteLock_Created = Lock_DateTime,
		@nvcExistingLock_User_WinLogon = Lock_User
	FROM dbo.tblRecord_Locks
	WHERE
			( Lock_Table = @nvcTableName )
		AND
			( Lock_Record_PK = @intRecord_PK )

	IF ( @bitLock_Exists = 1 ) BEGIN
		-- The lock exists.
		IF ( @nvcExistingLock_User_WinLogon = @nvcUser_WinLogon ) BEGIN
			-- The locking user and the one currently attempting to remove the lock are the same,
			-- hence allow removing the lock.
			SET @bitRemoveAllowed = 1
		END ELSE BEGIN
-- For this sample, there is no additional privileges that may be applied.
--			-- The locking user is not the one that currently attempts to remove the lock.
--			-- Check as to whether the unlocking user has administrative privileges.
--			-- Membership in the group "Admins" (ID = 1) is required.
--			SET @bitPrivilegesSufficient = dbo.f_Permissions_User_IsMemberOf_ByWinLogon(@nvcUser_WinLogon, 1)
--			
--			IF @bitPrivilegesSufficient = 1 BEGIN
--				-- Rights are sufficient - allow removing the lock.
--				SET @bitRemoveAllowed = 1
--			END ELSE BEGIN
				-- Rights are insufficient - prevent removing the lock and setup a message.
				SET @nvcNotRemoved_Reason = N'User ''' + @nvcUser_WinLogon + ''' is not the originator of the lock\n
					and does not have sufficient privileges to remove the lock.'
--			END
		END
	END ELSE BEGIN
		-- No lock present.
		SET @nvcNotRemoved_Reason = N'The requested record was not locked.'
		SET @bitLock_Removed = 0
	END


	-- If any of the conditions above indicated that the lock is to be removed, do so.
	IF @bitRemoveAllowed = 1 BEGIN
		DELETE 
		FROM dbo.tblRecord_Locks
		WHERE
				( Lock_Table = @nvcTableName )
			AND
				( Lock_Record_PK = @intRecord_PK )

		SET @bitLock_Removed = 1
		SET @bitLock_Exists = 0
	END

	-- Return info about the whole process
	SELECT 
		@nvcNotRemoved_Reason AS Lock_NotRemoved_Reason,
		@bitLock_Removed AS Lock_Removed,
		@bitLock_Exists AS Lock_StillExists 
RETURN
END
GO

/*

	Removes a record-lock identified by @Lock_GUID in the Lock Records table itself.

*/
CREATE PROCEDURE [dbo].[p_Lock_Attempt_Remove_ByLockGuid]
(
	@Lock_GUID uniqueidentifier			-- GUID of the record that is being locked
)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @nvcLock_User nvarchar(255)
	DECLARE @nvcLock_Table nvarchar(150)
	DECLARE @intLock_Record_PK int

	SELECT 
		@nvcLock_User = Lock_User,
		@nvcLock_Table = Lock_Table,
		@intLock_Record_PK = Lock_Record_PK 
	FROM dbo.tblRecord_Locks
	WHERE
			( Lock_GUID = @Lock_GUID )

	DELETE 
	FROM dbo.tblRecord_Locks
	WHERE
			( Lock_GUID = @Lock_GUID )

	-- If either the DELETE succeeded or the record didn't exist in the first place, the following
	-- will be returned as an indicator of whether the lock still exists.
	SELECT 
		Count(*) AS Locks_Remaining,
		@nvcLock_User AS Lock_User,
		@nvcLock_Table AS Lock_Table,
		@intLock_Record_PK AS Lock_Record_PK 
	FROM dbo.tblRecord_Locks
	WHERE
			( Lock_GUID = @Lock_GUID )
RETURN
END
GO

CREATE PROCEDURE dbo.p_Employees_LoadRecord
(
	@intEmployeeID int
)
AS
	SELECT * 
	FROM dbo.Employees
	WHERE ( EmployeeID = @intEmployeeID )
GO

CREATE PROCEDURE [dbo].[p_Employees_RecordList]
AS
	SELECT EmployeeID, COALESCE (LastName + N', ', N'') + COALESCE (FirstName, N'') AS Employee_Name, 
		LastName, FirstName, Title
	FROM dbo.Employees
	ORDER BY LastName, FirstName
GO

CREATE PROCEDURE [dbo].[p_Employees_DeleteRecord]
(
	@intEmployeeID int
)
AS
BEGIN
	SET NOCOUNT ON

	DECLARE @nvcLock_User nvarchar(255)
	DECLARE @nvcLock_Datetime datetime
	DECLARE @bitRecDeleted bit
	DECLARE @bitRecExists bit

	-- Standard: DS konnte nicht gelscht werden
	SET @bitRecDeleted = 0

	-- Standard: DS existiert nicht
	SET @bitRecExists = 0

	-- Prfen, ob der DS existiert
	SELECT @bitRecExists = CASE WHEN COUNT(*) > 0 THEN 1 ELSE 0 END
	FROM dbo.Employees
		WHERE
			( EmployeeID = @intEmployeeID )

	IF @bitRecExists = 1 BEGIN
		-- Prfen, ob der DS gesperrt ist
		SELECT @nvcLock_User = Lock_User, @nvcLock_Datetime = Lock_DateTime
		FROM tblRecord_Locks
		WHERE 
				( Lock_Table = 'Employees' )
			AND
				( Lock_Record_PK = @intEmployeeID )

		IF @nvcLock_User IS NULL BEGIN
			-- Dieser DS ist gegenwrtig nicht gesperrt und kann daher gelscht werden
			
			-- DS lschen
			DELETE FROM dbo.Employees
			WHERE
				( EmployeeID = @intEmployeeID )

			SET @bitRecDeleted = 1
		END
	END

	-- Rckgabe fr Auswertung durch Aufrufer
	SELECT 
		@bitRecExists AS RecordFound, 
		@bitRecDeleted AS RecordWasDeleted, 
		@nvcLock_User AS Lock_User, 
		@nvcLock_Datetime AS Lock_DateTime
	RETURN
END
GO

	
CREATE PROCEDURE [dbo].[p_Employees_SaveRecord]
(
	@intEmployeeID int,
   @nvcLastName nvarchar(50),
   @nvcFirstName nvarchar(50),
	@nvcTitle nvarchar(50)
)
AS
BEGIN
	DECLARE @bitNewRec bit

	-- Neuer DS?
	SELECT @bitNewRec = CASE WHEN Count(*) > 0 THEN 0 ELSE 1 END
	FROM dbo.Employees
	WHERE EmployeeID = @intEmployeeID

	IF @bitNewRec = 1 BEGIN
		-- Neuen DS einfgen
		INSERT INTO dbo.Employees (
			LastName,
			FirstName,
			Title
			) 
		VALUES (
			@nvcLastName,
			@nvcFirstName,
			@nvcTitle
			)

		SELECT scope_identity() AS New_ID
	END ELSE BEGIN
		-- Bestehenden DS aktualisieren
		UPDATE dbo.Employees
		SET
			LastName = @nvcLastName,
			FirstName = @nvcFirstName,
			Title = @nvcTitle
		WHERE EmployeeID = @intEmployeeID

		SELECT @intEmployeeID AS UpdatedID
	END
END

